Package: sccomp
Type: Package
Title: Differential Composition and Variability Analysis for
        Single-Cell Data
Version: 2.5.0
Date: 2024-01-15
Authors@R: c(person("Stefano", "Mangiola", email = "stefano.mangiola@unimelb.edu.au", role = c("aut", "cre")), person("Alexandra J.", "Roth-Schulze", role = "aut"), person("Marie", "Trussart", role = "aut"), person("Enrique", "Zozaya-Valdés", role = "aut"), person("Mengyao", "Ma", role = "aut"), person("Zijie", "Gao", role = "aut"), person("Alan F.", "Rubin", role = "aut"), person("Terence P.", "Speed", role = "aut"), person("Heejung", "Shim", role = "aut"), person("Anthony T.", "Papenfuss", role = "aut"))
Description: Comprehensive R package for differential composition and variability analysis in single-cell RNA sequencing, CyTOF, and microbiome data. Provides robust Bayesian modeling with outlier detection, random effects, and advanced statistical methods for cell type proportion analysis. Features include probabilistic outlier identification, mixed-effect modeling, differential variability testing, and comprehensive visualization tools. Perfect for cancer research, immunology, developmental biology, and single-cell genomics applications.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Depends: R (>= 4.3.0), instantiate (>= 0.2.3)
Imports: stats, boot, utils, scales, lifecycle, rlang, tidyselect,
        magrittr, crayon, cli, fansi, dplyr, tidyr, purrr, tibble,
        ggplot2, ggrepel, patchwork, forcats, readr, stringr, glue,
        SingleCellExperiment
Suggests: knitr, rmarkdown, BiocStyle, testthat (>= 3.0.0), markdown,
        loo, prettydoc, SeuratObject, tidyseurat,
        tidySingleCellExperiment, bayesplot, posterior
Additional_repositories: https://mc-stan.org/r-packages/
SystemRequirements: CmdStan
        (https://mc-stan.org/users/interfaces/cmdstan), C++14
biocViews: Bayesian, Regression, DifferentialExpression, SingleCell,
        Metagenomics, FlowCytometry, Spatial
LazyData: true
VignetteBuilder: knitr
URL: https://github.com/MangiolaLaboratory/sccomp
BugReports: https://github.com/MangiolaLaboratory/sccomp/issues
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/snapshot/parallel: true
Config/testthat/snapshot/parallel/workers: 2
git_url: https://git.bioconductor.org/packages/sccomp
git_branch: devel
git_last_commit: 6711013
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:53:15 UTC; biocbuild
Author: Stefano Mangiola [aut, cre],
  Alexandra J. Roth-Schulze [aut],
  Marie Trussart [aut],
  Enrique Zozaya-Valdés [aut],
  Mengyao Ma [aut],
  Zijie Gao [aut],
  Alan F. Rubin [aut],
  Terence P. Speed [aut],
  Heejung Shim [aut],
  Anthony T. Papenfuss [aut]
Maintainer: Stefano Mangiola <stefano.mangiola@unimelb.edu.au>
