% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{sccomp_theme}
\alias{sccomp_theme}
\title{Create a multipanel theme compatible with ggplot2 4.0.0 S7 system}
\usage{
sccomp_theme()
}
\value{
A \code{ggplot2} theme object compatible with ggplot2 4.0.0
}
\description{
Creates a multipanel theme that is compatible with the new ggplot2 4.0.0 S7 system.
This function creates a fresh theme object using the new S7 system instead of relying
on the saved S3 theme object.
}
