% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{handle_missing_values}
\alias{handle_missing_values}
\title{Handle missing values in a data frame}
\usage{
handle_missing_values(data)
}
\arguments{
\item{data}{A data frame to process}
}
\value{
A data frame with missing values handled:
\itemize{
\item For categorical variables (factor/character): NA values are converted to a factor level "NA"
\item For numeric variables: NA values are replaced with the mean of the column
}
}
\description{
Handle missing values in a data frame
}
